import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { DeleteResult, Repository } from 'typeorm';
import { Album } from './album.entity';
import { CreateAlbumDto } from './dto/create-album.dto';
import { UpdateAlbumDto } from './dto/update-album.dto';

@Injectable()
export class AlbumService {
  constructor(
    @InjectRepository(Album)
    private albumRepository: Repository<Album>
  ) {}

  findAll(): Promise<Album[]> {
    return this.albumRepository.find({
      order: {
        artist: "ASC",
        title: "ASC"
      }
    });
  }

  findOneById(id: number): Promise<Album | null> {
    return this.albumRepository.findOne({
      where: {
        id: id
      },
      relations: {
        songs: true
      }
    });
  }

  async create(createAlbumDto: CreateAlbumDto): Promise<Album | null> {
    const album = this.albumRepository.create({
      title: createAlbumDto.title,
      artist: createAlbumDto.artist,
      genre: createAlbumDto.genre
    });
    const savedAlbum = await this.albumRepository.save(album);
    return savedAlbum;
  }

  async update(id: number, updateAlbumDto: UpdateAlbumDto): Promise<Album | null> {
    console.log(id, updateAlbumDto);
    if (id == updateAlbumDto.id) {
      const albumToUpdate = await this.albumRepository.findOneBy({ id: updateAlbumDto.id });
      if (!albumToUpdate) {
        console.log("Didn't find album: ", id);
        return null;
      }

      albumToUpdate.title = updateAlbumDto.title;
      albumToUpdate.artist = updateAlbumDto.artist;
      albumToUpdate.genre = updateAlbumDto.genre;
      console.log("Saving album:", albumToUpdate);
      const savedAlbum = await this.albumRepository.save(albumToUpdate);
      console.log("Saved album: ", savedAlbum);
      return savedAlbum;
    } else {
      console.log("ids don't match", id, updateAlbumDto);
      return null;
    }
  }

  async remove(id: number): Promise<DeleteResult> {
    return await this.albumRepository.delete(id);
  }
}
